/* 
 * File: daemon.c  Created: 120225
 *
 * Copyright (c) 2012 Johannes Heimansberg (wejp.k.vu)
 *
 * Description: Daemon helper
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2 of
 * the License. See the file COPYING for details.
 */
#include <stdio.h>
#include <unistd.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>
#include <stdlib.h>
#include "daemon.h"
#include "messagequeue.h"

typedef void (*sighandler_t)(int);

static sighandler_t handle_signal(int sig_nr, sighandler_t sh)
{
	struct sigaction sig_new, sig_old;
	sighandler_t res = SIG_ERR;

	sig_new.sa_handler = sh;
	sigemptyset(&sig_new.sa_mask);
	sig_new.sa_flags = SA_RESTART;
	if (sigaction(sig_nr, &sig_new, &sig_old) >= 0)
		res = sig_old.sa_handler;
	return res;
}

void daemonize(const char *log_name, int facility)
{
	int   i;
	pid_t pid;

	/* Fork and close parent process... */
	if ((pid = fork()) != 0) exit(EXIT_FAILURE);

	if (setsid() < 0) {
		printf("setsid() failed. :(\n");
		exit(EXIT_FAILURE);
	}
	/* Ignore SIGHUP... */
	handle_signal(SIGHUP, SIG_IGN);
	/* Fork again and terminate child... */
	if ((pid = fork()) != 0) exit(EXIT_FAILURE);
	chdir("/");
	umask(0);
	/* Close all open filedescriptors... */
	for (i = sysconf(_SC_OPEN_MAX); i > 0; i--) close(i);
	openlog(log_name, LOG_PID | LOG_CONS | LOG_NDELAY, facility);
}

